unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, AppEvnts, ImgList, u_Sneak, Buttons, ExtCtrls, StdCtrls, ExtDlgs,
  GLSound, u_Score;

type
  TMainForm = class(TForm)
    Sprits: TImageList;
    AppliEvent: TApplicationEvents;
    GameZone: TImage;
    GameEvent: TTimer;
    Console: TPanel;
    StartGameBtn: TSpeedButton;
    ScoreLabel: TLabel;
    Pointlabel: TLabel;
    DecoImage: TImage;
    MenuBtn: TSpeedButton;
    OpenDlg: TOpenPictureDialog;
    PauseLabel: TLabel;
    Sounds: TGLSoundLibrary;
    procedure AppliEventMessage(var Msg: tagMSG;
      var Handled: Boolean);
    procedure StartGameBtnClick(Sender: TObject);
    procedure GameEventTimer(Sender: TObject);
    procedure MenuBtnClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure DecoImageClick(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure ShowMenu;
  end;


var
  MainForm: TMainForm;
  NewSnakeDir : TSnakeDir; //pour le synchronisme
  ScoreSize : String;      //sauvegarde pour les highscore
  ScoreSpeed : String;     //sauvegarde pour les highscore

implementation

uses Menu, HighScore, About;

{$R *.dfm}

//-----------------------------------------------------------
//Fonction d'affichage du menu et actualistion des paramtres
//-----------------------------------------------------------
procedure TMainForm.ShowMenu;
begin
MenuForm.ShowModal;
//choix du joueur
if MenuForm.Joueur1.Checked then Gamer:=0 else
if MenuForm.Joueur2.Checked then Gamer:=1 else
if MenuForm.Joueur3.Checked then Gamer:=2;

//choix de la vitesse
if MenuForm.Speed1.Checked then
  begin
  GameEvent.Interval :=200;
  ScoreSpeed := 'Lente';
  end else
if MenuForm.Speed2.Checked then
  begin
  GameEvent.Interval :=100;
  ScoreSpeed := 'Moyenne';
  end else
if MenuForm.Speed3.Checked then
  begin
  GameEvent.Interval :=50;
  ScoreSpeed := 'Rapide';
  end;

//choix de la taille du terrain
MainForm.Constraints.MaxWidth := 0;
MainForm.Constraints.MaxHeight := 0;
MainForm.Constraints.MinWidth := 0;
MainForm.Constraints.MinHeight := 0;

if MenuForm.Size1.Checked then
  begin
  MainForm.ClientWidth := SpriteSize*40;
  MainForm.ClientHeight := SpriteSize*20+Console.Height;
  ScoreSize := 'Petite';
  end else
if MenuForm.Size2.Checked then
  begin
  MainForm.ClientWidth := SpriteSize*55;
  MainForm.ClientHeight := SpriteSize*30+Console.Height;
  ScoreSize := 'Moyenne';
  end else
if MenuForm.Size3.Checked then
  begin
  MainForm.ClientWidth := SpriteSize*70;
  MainForm.ClientHeight := SpriteSize*40+Console.Height;
  ScoreSize := 'Grande';
  end;
GameZone.Picture := nil;
end;

//-----------------------------------------------------------
//Gestion des touches de dplacement et touche pause
//-----------------------------------------------------------
procedure TMainForm.AppliEventMessage(var Msg: tagMSG;
  var Handled: Boolean);
begin
    IF Msg.Message = WM_KEYDOWN then
    Case  Msg.wparam of
      VK_LEFT  : if SnakeDir <> E then NewSnakeDir := O;
      VK_RIGHT : if SnakeDir <> O then NewSnakeDir := E;
      VK_UP    : if SnakeDir <> S then NewSnakeDir := N;
      VK_DOWN  : if SnakeDir <> N then NewSnakeDir := S;
      VK_PAUSE : SnakePause(GameEvent,PauseLAbel,GameZone);
    end;
end;

//-----------------------------------------------------------
//Dmarre une partie
//-----------------------------------------------------------
procedure TMainForm.StartGameBtnClick(Sender: TObject);
begin
//initialisation
newSnakedir := S;
randomize;
ScoreOffset := AbsScoreMax - GameEvent.Interval;
inGame  := True;
//efface le terrain
GameZone.Picture := nil;
//Dmarre le jeux
InitSnake;
DrawSnake(GameZone,Sprits);
DrawNewTarget(GameZone,Sprits);
GameEvent.Enabled := true;
end;

procedure TMainForm.GameEventTimer(Sender: TObject);
var NowScore:TScore;
begin
//touche de direction synchrone
SnakeDir := NewSnakeDir;
//dplace le serpent
MoveSnake(GameZone,Sprits,Sounds,MenuForm.SoundBox.Checked);
//verifie si il y a gameover
if GameOver(GameZone) then
  begin
  GameEvent.Enabled := false;
  InGame := false;
  //enregistrement du highscore
  if IsHighScore(Score) then
    begin
    NowScore.Name := InputBox('HighScore','Entrez votre nom:','');
    NowScore.Score := Score;
    NowScore.Date := now;
    NowScore.Speed := ScoreSpeed;
    NowScore.Size := ScoreSize;
    NowScore.MaxPoint := MaxPoint;
    AddScore(NowScore);
    WriteAllScore;
    ScoreForm.Show;
    end else begin
    //si pas de highscore
    ShowMessage('GAME OVER !');
    end;
  end else begin
  //si pas gameover ractualisation de l'affichage
  DrawSnake(GameZone,Sprits);
  GameZone.Refresh;
  ScoreLabel.Caption := IntToStr(score);
  PointLabel.Caption := IntToStr(point);
  PointLabel.Left := MainForm.Width - PointLabel.Width - ScoreLabel.Width-20;
  end;
end;

//-----------------------------------------------------------
//Affichage du menu
//-----------------------------------------------------------
procedure TMainForm.MenuBtnClick(Sender: TObject);
begin
if not inGame then ShowMenu;
end;

//-----------------------------------------------------------
//blockage de la taille de la fenetre
//-----------------------------------------------------------
procedure TMainForm.FormShow(Sender: TObject);
begin
ShowMenu;
MainForm.Constraints.MaxHeight := MainForm.Height;
MainForm.Constraints.MinHeight := MainForm.Height;
MainForm.Constraints.MinWidth := MainForm.Width;
MainForm.Constraints.MaxWidth := MainForm.Width;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
ReadAllScore;
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
WriteAllScore;
end;

procedure TMainForm.DecoImageClick(Sender: TObject);
begin
AboutForm.Show;
end;

procedure TMainForm.FormKeyPress(Sender: TObject; var Key: Char);
begin
if key = #13 then
   StartGameBtn.Click;
end;

end.
